import sys

from nehushtan.cli.NehushtanArgumentParser import NehushtanArgumentParser

cli = NehushtanArgumentParser(sys.argv[1:]) \
    .add_option(key='Shovel Name', desc="The name of Shovel", short='s', long='shovel') \
    .add_option(key='Task ID', desc='The Task ID', short='t', long='task') \
    .add_option(key='Extra', desc='Extra Options', prefix='extra') \
    .parse()

print('usage')
print(cli.get_usage_text())
print('parsed')
print(cli.get_parsed_option_dict())
print('raw')
print(cli.get_raw_option_dict())



import time
from random import random

from nehushtan.logger.NehushtanFileLogger import NehushtanFileLogger
from nehushtan.queue.NehushtanQueueDelegate import NehushtanQueueDelegate
from nehushtan.queue.NehushtanQueueTask import NehushtanQueueTask
from nehushtan.queue.situation.NoNextTaskSituation import NoNextTaskSituation
from tests.queue.plan2.Test2NehushtanQueueTask import Test2NehushtanQueueTask


class Test2NehushtanQueueDelegate(NehushtanQueueDelegate):

    def __init__(self, config_dictionary: dict = None, logger: NehushtanFileLogger = None):
        super().__init__(config_dictionary, logger)
        self.last_task_id = 1000

    def when_loop_reports_error(self, error_message: str):
        self.logger.error('Test2NehushtanQueueDelegate when_loop_reports_error', error_message)

    def when_loop_terminates(self):
        self.logger.critical('Test2NehushtanQueueDelegate when_loop_terminates')

    def when_loop_should_not_run(self):
        self.logger.warning('Test2NehushtanQueueDelegate when_loop_should_not_run sleep 10')
        time.sleep(10)

    def check_next_task(self) -> NehushtanQueueTask:
        self.logger.info('Test2NehushtanQueueDelegate check_next_task')
        if random() > 0.7:
            self.last_task_id += 1
            self.logger.info('Test2NehushtanQueueDelegate check_next_task -> create new task')
            return Test2NehushtanQueueTask(task_id=self.last_task_id)
        else:
            raise NoNextTaskSituation('No task to execute')

    def when_no_task_to_do(self):
        self.logger.info('Test2NehushtanQueueDelegate when_no_task_to_do sleep 5')
        time.sleep(5)

    def when_task_not_executable(self, task: NehushtanQueueTask):
        self.logger.info('Test2NehushtanQueueDelegate when_task_not_executable', {'task_id': task.get_task_reference()})

    def when_to_execute_task(self, task: NehushtanQueueTask, pid: int):
        self.logger.info(
            'Test2NehushtanQueueDelegate when_to_execute_task',
            {'task_id': task.get_task_reference(), 'pid': pid}
        )

    def when_task_executed(self, task: NehushtanQueueTask, pid: int):
        self.logger.info(
            'Test2NehushtanQueueDelegate when_task_executed',
            {'task_id': task.get_task_reference(), 'pid': pid}
        )

    def when_task_raised_exception(self, task: NehushtanQueueTask, exception: Exception):
        self.logger.exception(
            f'Test2NehushtanQueueDelegate when_task_raised_exception task_id={task.get_task_reference()}',
            exception
        )



import email
import re
from email.header import decode_header
import email.parser

from nehushtan.helper.CommonHelper import CommonHelper
from nehushtan.mail.IMAPAgent import IMAPAgent
from tests.config import IMAP_CONFIG

status_name_array = (
    IMAPAgent.STATUS_NAME_MESSAGES,
    IMAPAgent.STATUS_NAME_RECENT,
    IMAPAgent.STATUS_NAME_UIDNEXT,
    IMAPAgent.STATUS_NAME_UIDVALIDITY,
    IMAPAgent.STATUS_NAME_UNSEEN,
)

imap_agent = IMAPAgent(IMAP_CONFIG['imap_server'], IMAP_CONFIG['imap_port'], True)
imap_agent.login(IMAP_CONFIG['username'], IMAP_CONFIG['password'])
boxes = imap_agent.list_mail_boxes()
for box in boxes:
    box_tuple = IMAPAgent.parse_box_string_to_tuple(box)
    # print(box,box_tuple)

    box_name = box_tuple[2]

    status = imap_agent.show_status(box_name, status_name_array)
    print('Box Status: ', status)

data = imap_agent.select_mailbox('INBOX')
print(f'Total Mails: {data}')

message_id_array = imap_agent.search_mails_in_mailbox('ALL')
print(message_id_array)

for message_id in message_id_array:
    print(f"MESSAGE [{message_id}]")
    data = imap_agent.fetch_mail(message_id, "(RFC822)")

    for i in range(len(data) - 1):
        for j in range(len(data[i])):
            print(i, j, data[i][j])

    header = CommonHelper.read_target(data, (0, 1), '')
    print('decoded header', header)
    # lines=header.splitlines()
    # for line in lines:
    #     line = line.strip()
    #     if line=='':
    #         continue
    #     parts=re.split(r': *',line.decode('gbk'),maxsplit=1)
    #     print(parts)

    email_parser = email.parser.BytesFeedParser()
    email_parser.feed(CommonHelper.read_target(data, (0, 1), b''))
    msg = email_parser.close()

    # print(msg)

    for header in ['subject', 'to', 'from']:
        print('{:^8}: {}'.format(header.upper(), IMAPAgent.decode_str(msg[header])))

    for part in msg.walk():
        part_info = {
            "content_type": part.get_content_type(),
            "file_name": part.get_filename(),
            "charset": part.get_charset(),
            "content_maintype": part.get_content_maintype(),
            "content_subtype": part.get_content_subtype(),
        }

        print('part info -> ', part_info)

imap_agent.logout()



from nehushtan.queue.NehushtanQueue import NehushtanQueue
from nehushtan.queue.NehushtanQueueDelegate import NehushtanQueueDelegate


class Test2NehushtanQueue(NehushtanQueue):
    def __init__(self, delegate: NehushtanQueueDelegate):
        super().__init__(delegate)



from nehushtan.helper.CommonHelper import CommonHelper

dictionary = {}

CommonHelper.write_dictionary(dictionary, tuple(['A', 'B', 'C']), 'X')
print(dictionary)

CommonHelper.write_dictionary(dictionary, tuple(['A', 'B', 'C']), 'Y')
print(dictionary)
